@@ -831,3 +831,81 @@ Gehan <- function() {
            },
            name = "Gehan loss")
 }
+
+
+### Hurdle model, negative binomial non-zero component with log link
+Hurdle <- function(nuirange = c(0, 100)){
+    sigma <- 1
+    plloss <- function(sigma, y, f)
+                  - (y * (f - log(1 / sigma + exp(f))) -
+                  (log(1 / sigma + exp(f)) + log(sigma)) / sigma +
+                  lgamma(y + 1 / sigma) - lgamma(y + 1) - lgamma(1 / sigma) -
+                  log(1 - (1 + exp(f) * sigma)^{-1 / sigma}))
+    riskS <- function(sigma, y, fit, w = 1)
+                 sum(w * plloss(y = y, f = fit, sigma = sigma))
+    risk <- function(y, f, w = 1) sum(w * plloss(y = y, f = f, sigma = sigma))
+    ngradient = function(y, f, w = 1){
+                    sigma <<- optimize(riskS, interval = nuirange, y = y,
+                                       fit = f, w = w)$minimum
+                    y / sigma / (exp(f) + 1 / sigma) - 1 / sigma /
+                    (exp(-f) / sigma + 1) - exp(f) * (1 + exp(f) *
+                    sigma)^{-1 / sigma - 1} / (1 - (1 + exp(f) *
+                    sigma)^{-1 / sigma})
+                    }
+    Family(ngradient = ngradient, risk = risk, check_y = function(y) {
+               stopifnot(all.equal(unique(y - floor(y)), 0))
+               y}, nuisance = function() return(sigma),
+               name = "Hurdle model, negative binomial non-zero part",
+               response = function(f) exp(f))
+} 
+
+### multinomial logit model
+### NOTE: this family can't be applied out-of-the box
+### the rhs for formula needs to read
+### ~ bl1 %O% bl0 + bl2 %O% bl1 + ...
+### where bl1 is some linear baselearner
+### and bl0 is a baselearner with design and penalty term
+### equal to the unit matrix for number of levels - 1
+### See ?Multinom
+Multinomial <- function() {
+    lev <- NULL
+    biny <- function(y) {
+        if (!is.factor(y))
+            stop("response is not a factor but ",
+                  sQuote("family = Multinomial()"))
+        lev <<- levels(y)
+        as.vector(model.matrix(~ y - 1)[,-length(lev)])
+    }
+    return(Family(ngradient = function(y, f, w = 1) {
+               if (length(f) != length(y)) 
+                   stop("predictor doesn't correspond to multinomial logit model; see ?Multinomial")
+               f <- pmin(abs(f), 36) * sign(f) 
+               p <- matrix(exp(f), ncol = length(lev) - 1)
+               p <- as.vector(p / (1 + rowSums(p)))
+               y - p  
+           },
+           loss = function(y, f) {
+               f <- pmin(abs(f), 36) * sign(f)
+               p <- matrix(exp(f), ncol = length(lev) - 1)
+               p <- as.vector(p / (1 + rowSums(p)))
+               -y * log(p) 
+           },
+           offset = function(y, w) {
+               return(rep(0, length(y)))
+           },
+           response = function(f) {
+               f <- pmin(abs(f), 36) * sign(f)
+               p <- matrix(exp(f), ncol = length(lev) - 1)
+               p <- cbind(p, 1) / (1 + rowSums(p))
+               colnames(p) <- lev
+               return(p)
+           },
+           rclass = function(f) {
+               f <- pmin(abs(f), 36) * sign(f)
+               p <- matrix(exp(f), ncol = length(lev) - 1)
+               p <- cbind(p, 1) / (1 + rowSums(p))
+               apply(p, 1, which.max)
+           },
+           check_y = biny,
+           name = "Negative Multinomial Likelihood"))
+}
